@@ -33,7 +33,7 @@ import ProxyPatternPool as ppp  # type: ignore
 # for local use & forwarding
 # NOTE the only missing should be "Flask"
 from flask import (
-    Response, Request, request, session, jsonify, Blueprint, make_response,
+    Response, Request, request, session, Blueprint, make_response,
     abort, redirect, url_for, after_this_request, send_file, current_app, g,
     send_from_directory, escape, Markup, render_template, get_flashed_messages,
     has_app_context, has_request_context, render_template_string,
@@ -82,15 +82,19 @@ class Mode(IntEnum):
     UNDEF = 0
     PROD = 1
     DEV = 2
-    DEBUG = 3
+    DEBUG1 = 3
     DEBUG2 = 4
     DEBUG3 = 5
+    DEBUG4 = 6
+    DEBUG = 3
 
 
 _MODES = {
-    "debug": Mode.DEBUG,
+    "debug1": Mode.DEBUG1,
     "debug2": Mode.DEBUG2,
     "debug3": Mode.DEBUG3,
+    "debug4": Mode.DEBUG4,
+    "debug": Mode.DEBUG,
     "dev": Mode.DEV,
     "prod": Mode.PROD,
 }
@@ -186,10 +190,23 @@ def _get_file_storage(p: str) -> FileStorage:
     return request.files[p]
 
 
+def jsonify(a: Any):
+    """Jsonify something, including generators."""
+    # FIXME how to detect a generator?
+    if inspect.isgenerator(a) or type(a) in (map, filter, range):
+        return flask.jsonify(list(a))
+    else:
+        return flask.jsonify(a)
+
+
 class Reference(ppp.Proxy):
     """Convenient object wrapper class."""
 
-    def __init__(self, *args, close: Optional[str] = "close", **kwargs):
+    def __init__(self, *args, close: Optional[str] = "close", max_delay: float = None, **kwargs):
+        # NOTE max_delay is just here for backward compatibility
+        # TODO remove at some point
+        if max_delay is not None and ppp.__version__ >= (4, 0):  # pragma: no cover
+            kwargs["max_avail_delay"] = max_delay
         super().__init__(*args, close=close, **kwargs)
 
 
@@ -349,6 +366,8 @@ class FlaskSimpleAuth:
         self._after_requests: List[AfterRequestFun] = []
         self._local: Any = None
         # registered here to avoid being bypassed by user hooks
+        # FIXME not always?
+        self._app.before_request(self._show_request)
         self._app.before_request(self._auth_reset_user)
         self._app.before_request(self._check_secure)
         self._app.before_request(self._run_before_requests)
@@ -377,8 +396,10 @@ class FlaskSimpleAuth:
             log.debug(f"error: {code} {msg}")
         return self._Exc(exc) or ErrorResponse(msg, code)
 
-    def _Bad(self, msg: str):
+    def _Bad(self, msg: str, misc: str = None):
         """Build and trace an exception on a bad configuration."""
+        if misc:
+            msg += "\n" + misc
         log.critical(msg)
         return ConfigError(msg)
 
@@ -405,6 +426,23 @@ class FlaskSimpleAuth:
                     log.warning("insecure HTTP request seen")
                     self._secure_warning = False
 
+    def _show_request(self):
+        """Show request."""
+        if self._mode >= Mode.DEBUG4:
+            # FIXME there is no decent request prettyprinter
+            r = request
+            rpp = f"{r}\n"
+            params = self._params()
+            if params:
+                rpp += " - params: " + ", ".join(sorted(params.keys())) + "\n"
+            if request.files:
+                rpp += " - files: " + ", ".join(sorted(request.files.keys())) + "\n"
+            if not params and not request.files:
+                rpp += " - no params, no files\n"
+            rpp += f"\t{r.method} {r.path} HTTP/?\n\t" + \
+                "\n\t".join(f"{k}: {v}" for k, v in r.headers.items()) + "\n"
+            log.debug(rpp)
+
     def _auth_reset_user(self):
         """Before request hook to cleanup authentication and authorization."""
         self._local.start = dt.datetime.timestamp(dt.datetime.now())
@@ -508,11 +546,21 @@ class FlaskSimpleAuth:
     def _add_fsa_headers(self, res: Response):
         """Add convenient FSA-related headers."""
         res.headers["FSA-Request"] = f"{request.method} {request.path}"
-        res.headers["FSA-User"] = self.current_user()
+        res.headers["FSA-User"] = f"{self.current_user()} ({self._local.source})"
         delay = dt.datetime.timestamp(dt.datetime.now()) - self._local.start
         res.headers["FSA-Delay"] = f"{delay:.6f}"
         return res
 
+    def _show_response(self, res: Response):
+        """Show response."""
+        if self._mode >= Mode.DEBUG4:
+            # FIXME there is no decent response prettyprinter
+            r = res
+            rpp = (f"{r}\n\tHTTP/? {r.status}\n\t" +
+                   "\n\t".join(f"{k}: {v}" for k, v in r.headers.items()) + "\n")
+            log.debug(rpp)
+        return res
+
     def _params(self):
         """Get request parameters wherever they are."""
         if request.is_json:
@@ -966,6 +1014,8 @@ class FlaskSimpleAuth:
         #
         self._before_requests = conf.get("FSA_BEFORE_REQUEST", [])
         # internal hooks
+        if self._mode >= Mode.DEBUG4:
+            self._app.after_request(self._show_response)
         if self._mode >= Mode.DEV:
             self._app.after_request(self._add_fsa_headers)
         self._after_requests = conf.get("FSA_AFTER_REQUEST", [])
@@ -1458,7 +1508,8 @@ class FlaskSimpleAuth:
                 self._Exc(e)
 
         # even if not set, we say that the answer is the right one.
-        self._local.source = "none"
+        if not self._local.source:
+            self._local.source = "none"
 
         # rethrow last auth exception on failure
         if required and not self._local.user:
@@ -1505,7 +1556,7 @@ class FlaskSimpleAuth:
     #  _oauth_authz: check OAuth scope authorization
     #  _group_authz: check group authorization
     #     _no_authz: validate that no authorization was needed
-    #   _parameters: handle HTTP/JSON to python parameter translation
+    #   _parameters: handle HTTP/JSON/FILE to python parameter translation
     #     _noparams: just check that no parameters are passed
     #   _perm_authz: check per-object permissions
     #
@@ -1674,6 +1725,8 @@ class FlaskSimpleAuth:
             # parameters types/casts and defaults taken from signature
             sig, keywords = inspect.signature(fun), False
 
+            where = f"{fun.__name__}() at {fun.__code__.co_filename}:{fun.__code__.co_firstlineno}"
+
             # build helpers
             for n, p in sig.parameters.items():
                 sn = n[1:] if n[0] == "_" and len(n) > 1 else n
@@ -1683,6 +1736,15 @@ class FlaskSimpleAuth:
                     t = _typeof(p)
                     types[n] = t
                     typings[n] = self._casts.get(t, t)
+                    # check that type can be isinstance and is castable
+                    try:
+                        isinstance("", t)
+                    except TypeError as e:
+                        raise self._Bad(f"parameter {n} type {t} is not (yet) supported: {e}", where)
+                    except Exception as e:  # pragma: no cover
+                        raise self._Bad(f"parameter {n} type {t} is not (yet) supported: {e}", where)
+                    if not callable(typings[n]) or typings[n].__module__ == "typing":
+                        raise self._Bad(f"parameter {n} type cast {typings[n]} is not callable", where)
                 if p.default != inspect._empty:
                     defaults[n] = p.default
                 if p.kind == p.VAR_KEYWORD:
@@ -1697,6 +1759,7 @@ class FlaskSimpleAuth:
             signature = f"{fun.__name__}({', '.join(mandatory)}, [{', '.join(optional)}])"
 
             def debugParam():
+                # FIXME params = sorted(self._params().keys() + request.files.keys())
                 params = sorted(self._params().keys())
                 mtype = request.headers.get("Content-Type", "?")
                 return f"{signature}: {' '.join(params)} [{mtype}]"
