@@ -1,5 +1,6 @@
 # FlaskSimpleAuth Demo
 
+import uuid
 import logging
 logging.basicConfig()
 
@@ -39,11 +40,19 @@ def get_now():
 def get_who():
     return json(app.get_user(required=False)), 200
 
+
+# POST /upload (file): upload a file!
+@app.post("/upload", authorize="ALL")
+def post_upload(file: fsa.FileStorage):
+    filename = str(uuid.uuid4()) + ".tmp"
+    file.save(app.config["APP_UPLOAD_DIR"] + "/" + filename)
+    return f"{file.filename} saved as {filename}", 201
+
+
 #
 # then register some blueprints
 #
 
-
 # stuff management by users
 from stuff import stuff
 app.register_blueprint(stuff)
