@@ -5,6 +5,8 @@
 
 import io
 import re
+from typing import List
+
 import pytest
 import App
 from App import app
@@ -897,46 +899,74 @@ def test_per_route(client):
     TOKEN = {"Authorization": f"Bearer {token}"}
     # basic
     log.debug("trying: basic")
-    check(200, client.get("/auth/basic", headers=BASIC))
+    res = check(200, client.get("/auth/basic", headers=BASIC))
+    assert "basic auth: calvin" in res.text
+    assert "calvin (basic)" in res.headers["FSA-User"]
     check(401, client.get("/auth/basic", headers=TOKEN))
     check(401, client.get("/auth/basic", data=PARAM))
     check(401, client.get("/auth/basic", json=PARAM))
     check(401, client.get("/auth/basic", data=FAKE))
     check(401, client.get("/auth/basic", json=FAKE))
     # param
-    check(200, client.get("/auth/param", data=PARAM))
-    check(200, client.get("/auth/param", json=PARAM))
+    res = check(200, client.get("/auth/param", data=PARAM))
+    assert "param auth: calvin" in res.text
+    assert "calvin (param)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/param", json=PARAM))
+    assert "param auth: calvin" in res.text
+    assert "calvin (param)" in res.headers["FSA-User"]
     check(401, client.get("/auth/param", headers=BASIC))
     check(401, client.get("/auth/param", headers=TOKEN))
     check(401, client.get("/auth/param", data=FAKE))
     check(401, client.get("/auth/param", json=FAKE))
     # password
-    check(200, client.get("/auth/password", headers=BASIC))
-    check(200, client.get("/auth/password", data=PARAM))
-    check(200, client.get("/auth/password", json=PARAM))
+    res = check(200, client.get("/auth/password", headers=BASIC))
+    assert "password auth: calvin" in res.text
+    assert "calvin (password)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/password", data=PARAM))
+    assert "password auth: calvin" in res.text
+    assert "calvin (password)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/password", json=PARAM))
+    assert "password auth: calvin" in res.text
+    assert "calvin (password)" in res.headers["FSA-User"]
     check(401, client.get("/auth/password", headers=TOKEN))
     check(401, client.get("/auth/password", data=FAKE))
     check(401, client.get("/auth/password", json=FAKE))
     # token
-    check(200, client.get("/auth/token", headers=TOKEN))
+    res = check(200, client.get("/auth/token", headers=TOKEN))
+    assert "token auth: calvin" in res.text
+    assert "calvin (token)" in res.headers["FSA-User"]
     check(401, client.get("/auth/token", data=PARAM))
     check(401, client.get("/auth/token", json=PARAM))
     check(401, client.get("/auth/token", headers=BASIC))
     check(401, client.get("/auth/token", data=FAKE))
     check(401, client.get("/auth/token", json=FAKE))
     # fake
-    check(200, client.get("/auth/fake", data=FAKE))
-    check(200, client.get("/auth/fake", json=FAKE))
+    res = check(200, client.get("/auth/fake", data=FAKE))
+    assert "fake auth: calvin" in res.text
+    assert "calvin (fake)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/fake", json=FAKE))
+    assert "fake auth: calvin" in res.text
+    assert "calvin (fake)" in res.headers["FSA-User"]
     check(401, client.get("/auth/fake", headers=TOKEN))
     check(401, client.get("/auth/fake", data=PARAM))
     check(401, client.get("/auth/fake", json=PARAM))
     check(401, client.get("/auth/fake", headers=BASIC))
     # fake, token, param
-    check(200, client.get("/auth/ftp", data=FAKE))
-    check(200, client.get("/auth/ftp", json=FAKE))
-    check(200, client.get("/auth/ftp", headers=TOKEN))
-    check(200, client.get("/auth/ftp", data=PARAM))
-    check(200, client.get("/auth/ftp", json=PARAM))
+    res = check(200, client.get("/auth/ftp", data=FAKE))
+    assert "ftp auth: calvin" in res.text
+    assert "calvin (fake)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/ftp", json=FAKE))
+    assert "ftp auth: calvin" in res.text
+    assert "calvin (fake)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/ftp", headers=TOKEN))
+    assert "ftp auth: calvin" in res.text
+    assert "calvin (token)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/ftp", data=PARAM))
+    assert "ftp auth: calvin" in res.text
+    assert "calvin (param)" in res.headers["FSA-User"]
+    res = check(200, client.get("/auth/ftp", json=PARAM))
+    assert "ftp auth: calvin" in res.text
+    assert "calvin (param)" in res.headers["FSA-User"]
     check(401, client.get("/auth/ftp", headers=BASIC))
 
 def test_bad_app():
@@ -1702,11 +1732,26 @@ def test_request_hooks():
     res = check(555, client.get("/cool"))
     assert res.data == b"Ooops!"
 
+def hello(app: fsa.Flask):
+    @app.get("/hello", authorize="ANY")
+    def get_hello(name: str):
+        return f"Hello {name}!", 200
+    with app.test_client() as c:
+        res = check(200, c.get("/hello", data={"name": "Calvin"}))
+        assert res.data == b"Hello Calvin!"
+        check(400, c.get("/hello", data={"nom": "Calvin"}))
+        check(400, c.get("/hello", data={"name": "Calvin", "nom": "Hobbes"}))
+        res = check(200, c.get("/hello", json={"name": "Hobbes"}))
+        assert res.data == b"Hello Hobbes!"
+        check(400, c.get("/hello", json={"nom": "Hobbes"}))
+        check(400, c.get("/hello", json={"nom": "Calvin", "name": "Hobbes"}))
+
 def test_mode():
+    # combine all debug/modes
     for debug in (True, False):
-        for mode in ("debug3", "debug2", "debug", "dev", "prod"):
-            app = fsa.Flask("mode", debug=debug, FSA_MODE=mode)
-            app = fsa.Flask("mode", FSA_DEBUG=debug, FSA_MODE=mode)
+        for mode in ("debug4", "debug3", "debug2", "debug1", "debug", "dev", "prod"):
+            hello(fsa.Flask("mode", debug=debug, FSA_MODE=mode))
+            hello(fsa.Flask("mode", FSA_DEBUG=debug, FSA_MODE=mode))
     try:
         app = fsa.Flask("mode", FSA_MODE="unexpected")
         app._fsa.initialize()
@@ -1755,7 +1800,7 @@ def test_user_errors():
 
 def test_param_params():
     app = fsa.Flask("pp",
-        FSA_MODE="debug2",
+        FSA_MODE="debug4",
         FSA_DEBUG_LEVEL=logging.DEBUG,
         FSA_AUTH="param",
         FSA_PARAM_USER="login",
@@ -1812,7 +1857,7 @@ def test_file_storage():
     def bfile(contents: bytes, name: str = "foo.txt", ct: str = "text/plain"):
         return (io.BytesIO(contents), name, ct)
 
-    app = fsa.Flask("file-storage", FSA_MODE="debug2")
+    app = fsa.Flask("file-storage", FSA_MODE="debug4")
 
     @app.post("/upload", authorize="ANY")
     def post_upload(file: fsa.FileStorage):
@@ -1857,3 +1902,51 @@ def test_file_storage():
     assert b"nothing" in res.data
     res = check(400, client.post("/empty", data={"foo": bfile(b"hello foo!\n"), "bla": bfile(b"hello bla!\n")}))
     assert b"unexpected file parameters: bla foo" in res.data
+
+
+def test_param_types():
+    app = fsa.Flask("param-types", FSA_MODE="debug")
+
+    try:
+        @app.post("/list-str", authorize="ANY")
+        def post_list_str(ls: List[str]):
+            return json(len(ls)), 201
+        assert False, "should raise a config error"
+    except ConfigError as e:
+        assert "not (yet) supported" in str(e)
+
+    try:
+        @app.post("/list", authorize="ANY")
+        def post_list(l: List):
+            return json(len(l)), 201
+        assert False, "should raise a config error"
+    except ConfigError as e:
+        assert "is not callable" in str(e)
+
+
+def test_jsonify_with_generators():
+    def gen(i: int):
+        for i in range(i):
+            yield i
+    app = fsa.Flask("json-gen")
+    @app.get("/json", authorize="ANY")
+    def get_json(what: str):
+        l = [0, 1, 2]
+        res = (l if what == "list" else
+               map(lambda i: i+1, l) if what == "map" else
+               filter(lambda i: i>0, l) if what == "filter" else
+               range(2, 3) if what == "range" else
+               gen(2) if what == "gen" else
+               None)
+        return fsa.jsonify(res), 200
+    with app.test_client() as c:
+        res = check(200, c.get("/json", data={"what": "list"}))
+        assert res.data == b"[0,1,2]\n"
+        res = check(200, c.get("/json", data={"what": "map"}))
+        assert res.data == b"[1,2,3]\n"
+        res = check(200, c.get("/json", data={"what": "filter"}))
+        assert res.data == b"[1,2]\n"
+        res = check(200, c.get("/json", data={"what": "range"}))
+        assert res.data == b"[2]\n"
+        res = check(200, c.get("/json", data={"what": "gen"}))
+        assert res.data == b"[0,1]\n"
